==> rrdtool_1.0.35.orig.tar.gz: rrdtool-1.0.35/src/rrd_diff.c <==
/*****************************************************************************
 * RRDtool 1.0.35  Copyright Tobias Oetiker, 1999
 * This code is stolen from rateup (mrtg-2.x) by Dave Rand
 *****************************************************************************
 * diff calculate the difference between two very long integers available as
 *      strings
 *****************************************************************************
 * $Id: rrd_diff.c,v 1.1.1.1 2002/02/26 10:21:37 oetiker Exp $
 * $Log: rrd_diff.c,v $
 * Revision 1.1.1.1  2002/02/26 10:21:37  oetiker
 * Intial Import
 *
 * Revision 1.1  1998/10/08 18:21:45  oetiker
 * Initial revision
 *
 * Revision 1.3  1998/02/06 21:10:52  oetiker
 * removed max define .. it is now in rrd_tool.h
 *
 * Revision 1.2  1997/12/07 20:38:03  oetiker
 * ansified
 *
 * Revision 1.1  1997/11/28 23:31:59  oetiker
 * Initial revision
 *
 *****************************************************************************/

#include "rrd_tool.h"

double
rrd_diff(char *a, char *b)
{
    char res[LAST_DS_LEN+1], *a1, *b1, *r1, *fix;
    int c,x,m;
    
    while (!isdigit((int)*a) || *a==0)
        a++;
    fix=a;
    while (isdigit((int)*fix)) 
	fix++;
    *fix = 0; /* maybe there is some non digit data in the string */ 
    while (!isdigit((int)*b) || *b==0)
        b++;
    fix=b;
    while (isdigit((int)*fix)) 
	fix++;
    *fix = 0; /* maybe there is some non digit data in the string */ 
    if(!isdigit((int)*a) || !isdigit((int)*b))
	return DNAN;
    a1 = &a[strlen(a)-1];
    m = max(strlen(a),strlen(b));
    if (m > LAST_DS_LEN) return DNAN; /* result string too short */

    r1 = &res[m+1];
    for (b1 = res;b1 <= r1; b1++) *b1 = ' ';
    b1 = &b[strlen(b)-1];
    r1[1] = 0;  /* Null terminate result */
    c = 0;
    for (x=0; x<m; x++) {
        if (a1 >= a && b1 >= b) {
            *r1 = ((*a1 - c) - *b1) + '0';
        } else if (a1 >= a) {
            *r1 = (*a1 - c);
        } else {
            *r1 = ('0' - *b1 - c) + '0';
        }
        if (*r1 < '0') {
            *r1 += 10;
            c=1;
        } else
	  if (*r1 > '9') { /* 0 - 10 */
	    *r1 -= 10;
	    c=1;	    
	  } else {
            c=0;
        }
        a1--;b1--;r1--;
    }
    if (c) {
        r1 = &res[m+1];
        for (x=0; isdigit((int)*r1) && x<m; x++,r1--)  {
            *r1 = ('9' - *r1 + c) + '0';
            if (*r1 > '9') {
                *r1 -= 10;
                c=1;
            } else {
                c=0;
            }
        }
        return(-atof(res));
    } else
        return(atof(res));
}

==> rrdtool-1.0.37.tar.gz: rrdtool-1.0.37/src/rrd_diff.c <==
/*****************************************************************************
 * RRDtool 1.0.37  Copyright Tobias Oetiker, 1999
 * This code is stolen from rateup (mrtg-2.x) by Dave Rand
 *****************************************************************************
 * diff calculate the difference between two very long integers available as
 *      strings
 *****************************************************************************
 * $Id: rrd_diff.c,v 1.1.1.1 2002/02/26 10:21:37 oetiker Exp $
 * $Log: rrd_diff.c,v $
 * Revision 1.1.1.1  2002/02/26 10:21:37  oetiker
 * Intial Import
 *
 * Revision 1.1  1998/10/08 18:21:45  oetiker
 * Initial revision
 *
 * Revision 1.3  1998/02/06 21:10:52  oetiker
 * removed max define .. it is now in rrd_tool.h
 *
 * Revision 1.2  1997/12/07 20:38:03  oetiker
 * ansified
 *
 * Revision 1.1  1997/11/28 23:31:59  oetiker
 * Initial revision
 *
 *****************************************************************************/

#include "rrd_tool.h"

double
rrd_diff(char *a, char *b)
{
    char res[LAST_DS_LEN+1], *a1, *b1, *r1, *fix;
    int c,x,m;
    
    while (!(isdigit((int)*a) || *a==0))
        a++;
    fix=a;
    while (isdigit((int)*fix)) 
	fix++;
    *fix = 0; /* maybe there is some non digit data in the string */ 
    while (!(isdigit((int)*b) || *b==0))
        b++;
    fix=b;
    while (isdigit((int)*fix)) 
	fix++;
    *fix = 0; /* maybe there is some non digit data in the string */ 
    if(!isdigit((int)*a) || !isdigit((int)*b))
	return DNAN;
    a1 = &a[strlen(a)-1];
    m = max(strlen(a),strlen(b));
    if (m > LAST_DS_LEN) return DNAN; /* result string too short */

    r1 = &res[m+1];
    for (b1 = res;b1 <= r1; b1++) *b1 = ' ';
    b1 = &b[strlen(b)-1];
    r1[1] = 0;  /* Null terminate result */
    c = 0;
    for (x=0; x<m; x++) {
        if (a1 >= a && b1 >= b) {
            *r1 = ((*a1 - c) - *b1) + '0';
        } else if (a1 >= a) {
            *r1 = (*a1 - c);
        } else {
            *r1 = ('0' - *b1 - c) + '0';
        }
        if (*r1 < '0') {
            *r1 += 10;
            c=1;
        } else
	  if (*r1 > '9') { /* 0 - 10 */
	    *r1 -= 10;
	    c=1;	    
	  } else {
            c=0;
        }
        a1--;b1--;r1--;
    }
    if (c) {
        r1 = &res[m+1];
        for (x=0; isdigit((int)*r1) && x<m; x++,r1--)  {
            *r1 = ('9' - *r1 + c) + '0';
            if (*r1 > '9') {
                *r1 -= 10;
                c=1;
            } else {
                c=0;
            }
        }
        return(-atof(res));
    } else
        return(atof(res));
}
